@@ -83,7 +83,9 @@ it will happen soon. *)
     mutable op_file_to_option : ('a -> (string * option_value) list);
     mutable op_file_cancel : ('a -> unit);
     mutable op_file_pause : ('a -> unit);
+    mutable op_file_queue : ('a -> unit);
     mutable op_file_resume : ('a -> unit);
+    mutable op_file_download_order : ('a -> swarming_strategy option -> swarming_strategy option);
     mutable op_file_info : ('a -> GuiTypes.file_info);
     mutable op_file_set_format : ('a -> CommonTypes.format -> unit);
     mutable op_file_check : ('a -> unit);
@@ -91,7 +93,6 @@ it will happen soon. *)
     mutable op_file_all_sources : ('a -> client list);
     mutable op_file_active_sources : ('a -> client list);
     mutable op_file_comment : ('a -> string);
-    mutable op_file_set_priority : ('a -> int -> unit);
     mutable op_file_print : ('a -> CommonTypes.ui_conn -> unit);
     mutable op_file_print_sources : ('a -> CommonTypes.ui_conn -> unit);
     mutable op_file_files : ('a -> 'a file_impl -> file list);    
@@ -140,8 +141,8 @@ let dummy_file_impl = {
     impl_file_last_seen = 0;
     impl_file_comment = "";
     impl_file_probable_name = None;
-    impl_file_owner = admin_user;
-    impl_file_group = Some system_user_default_group;
+    impl_file_owner = admin_user ();
+    impl_file_group = Some (admin_group ());
   }
 
 let dummy_file = as_file dummy_file_impl
@@ -245,11 +246,19 @@ let file_pause (file : file) user =
       file.impl_file_ops.op_file_pause file.impl_file_val
   | _ -> ()
 
+let file_queue (file : file) =
+  let file = as_file_impl file in
+  match file.impl_file_state with
+  | FileDownloading ->
+      update_file_state file FileQueued;
+      file.impl_file_ops.op_file_queue file.impl_file_val
+  | _ -> ()
+
 let file_resume (file : file) user =
   if user2_allow_file_admin file user then
   let file = as_file_impl file in
   match file.impl_file_state with
-  | FilePaused | FileAborted _ ->
+  | FileQueued | FilePaused | FileAborted _ ->
       update_file_state file FileDownloading;
       file.impl_file_ops.op_file_resume file.impl_file_val
   | _ -> ()
@@ -312,7 +321,8 @@ let set_file_best_name file name fs namemax =
     if name <> real_name then
       lprintf_nl "wanted new name \"%s\" changed to \"%s\" due to system limitations"
         (String.escaped name) (String.escaped file.impl_file_best_name);
-    if !verbose && old_name <> file.impl_file_best_name then
+    if !verbose && old_name <> file.impl_file_best_name &&
+       (not !CommonGlobals.is_startup_phase) then
       lprintf_nl "best_name of \"%s\" changed to \"%s\""
         (String.escaped old_name) (String.escaped file.impl_file_best_name)
   end
@@ -399,8 +409,8 @@ let default_file_print_sources file o =
           ("", "sr br", addr);
           (GuiTypes.client_software cinfo.GuiTypes.client_software cinfo.GuiTypes.client_os,
 	   "sr br", GuiTypes.client_software_short cinfo.GuiTypes.client_software cinfo.GuiTypes.client_os);
-          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_uploaded));
-          ("", "sr ar br", (size_of_int64 cinfo.GuiTypes.client_downloaded)); ];
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_total_uploaded));
+          ("", "sr ar br", (size_of_int64 cinfo.GuiTypes.client_total_downloaded)); ];
 
         Printf.bprintf buf "\\</tr\\>";
 
@@ -521,7 +531,6 @@ let set_file_priority file p =
   let impl = as_file_impl file in
   if impl.impl_file_priority <> p then begin
       impl.impl_file_priority <- p;
-      impl.impl_file_ops.op_file_set_priority  impl.impl_file_val p;
       file_must_update file
     end
 
@@ -557,6 +566,19 @@ let file_preview (file : file) =
       [| Filename2.basename !!previewer; file_disk_name file; file_best_name file |]
       Unix.stdin Unix.stdout Unix.stderr)
 
+let file_download_order (file : file) strategy =
+  let file = as_file_impl file in
+  file.impl_file_ops.op_file_download_order file.impl_file_val strategy
+
+let file_print_download_order (file : file) =
+  let file = as_file_impl file in
+  match file.impl_file_ops.op_file_download_order file.impl_file_val None with
+  | None -> "unknown"
+  | Some strategy ->
+      match strategy with
+      | LinearStrategy -> "linear"
+      | AdvancedStrategy -> "random"
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         file_downloaders                              *)
@@ -762,6 +784,11 @@ parent.fstatus.location.href='submit?q=chgrp+'+v+'+%d';
           Some group -> Printf.sprintf "%s" group.group_name
         | None -> "None"))];
 
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+      html_mods_td buf [
+        ("Download strategy", "sr br", "DL strategy");
+        ("", "sr", file_print_download_order file) ];
+
       Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
       html_mods_td buf [
         ("Number of file sources", "sr br", "Sources");
@@ -987,7 +1014,9 @@ let new_file_ops network =
       op_file_info = (fun _ -> fni network "file_info");
       op_file_files = (fun _ impl -> [as_file impl]);
       op_file_pause = (fun _ -> ni_ok network "file_pause");
+      op_file_queue = (fun _ -> ni_ok network "file_queue");
       op_file_resume = (fun _ -> ni_ok network "file_resume");
+      op_file_download_order = (fun _ _ -> None);
 (*      op_file_disk_name = (fun _ -> fni network "file_disk_name"); *)
       op_file_check = (fun _ -> ni_ok network "file_check");
       op_file_recover = (fun _ -> ni_ok network "file_recover");
@@ -995,7 +1024,6 @@ let new_file_ops network =
       op_file_all_sources = (fun _ -> fni network "file_all_sources");
       op_file_active_sources = (fun _ -> fni network "file_active_sources");
       op_file_comment = (fun _ -> ni_ok network "file_comment"; "");
-      op_file_set_priority = (fun _ _ -> ni_ok network "file_set_priority");
       op_file_print = (fun _ _ -> ni_ok network "file_print_html");
       op_file_print_sources = (fun _ _ -> fni network "file_print_sources");
       op_file_debug = (fun _ -> "");
@@ -1026,8 +1054,12 @@ let check_file_implementations () =
         lprintf_nl "op_file_cancel";
       if c.op_file_pause == cc.op_file_pause then
         lprintf_nl "op_file_pause";
+      if c.op_file_queue == cc.op_file_queue then
+        lprintf_nl "op_file_queue";
       if c.op_file_resume == cc.op_file_resume then
         lprintf_nl "op_file_resume";
+      if c.op_file_download_order == cc.op_file_download_order then
+        lprintf_nl "op_file_download_order";
 (*      if c.op_file_disk_name == cc.op_file_disk_name then
         lprintf_nl "op_file_disk_name"; *)
       if c.op_file_check == cc.op_file_check then
