@@ -103,7 +103,7 @@ let rec dollar_escape o with_frames s =
 
 let eval auth cmd o =
   let buf = o.conn_buf in
-  let cmd = Url.decode cmd in
+  let cmd = Url.decode ~raw:false cmd in
   let cmd =
     if String2.check_prefix cmd "ed2k://" ||
        String2.check_prefix cmd "ftp://" ||
@@ -285,7 +285,7 @@ $bGeneral:$n
 Use '$rlonghelp$n' or '$r??$n' for all commands.
 Use '$rhelp command$n' or '$r? command$n' for help on a command.
             ";
-    | "?" :: args | "help" :: args ->
+    | "?" :: args | "help" :: args | "man" :: args ->
           List.iter (fun arg ->
               List.iter (fun (cmd, _, _, help) ->
                   if cmd = arg then
@@ -310,7 +310,7 @@ Use '$rhelp command$n' or '$r? command$n' for help on a command.
         let user, pass =
           match args with
             [] -> failwith "Usage: auth <user> <password>"
-          | [s1] -> admin_user.CommonTypes.user_name, s1
+          | [s1] -> (admin_user ()).CommonTypes.user_name, s1
           | user :: pass :: _ -> user, pass
         in
         if valid_password user pass then begin
@@ -342,8 +342,9 @@ let calendar_options = {
     conn_output = TEXT;
     conn_sortvd = NotSorted;
     conn_filter = (fun _ -> ());
-    conn_user = default_user;
+    conn_user = find_ui_user CommonUserDb.admin_user_name;
     conn_width = 80; conn_height = 0;
+    conn_info = Some (CALENDAR, (Ip.null, 0));
   }
 
 let check_calendar () =
@@ -452,7 +453,7 @@ type telnet_conn = {
     telnet_auth : bool ref;
   }
 
-
+let iac_will_8bit = "\255\253\000"
 let iac_will_naws = "\255\253\031"
 
 let user_reader o telnet sock nread =
@@ -480,7 +481,7 @@ let user_reader o telnet sock nread =
 
       let i = int_of_char c in
       telnet.telnet_iac <- false;
-      let is_normal_char = i > 31 && i < 127 in
+      let is_normal_char = i > 31 in
 
       if telnet.telnet_wait = 1 then begin
           Buffer.add_char telnet.telnet_buffer c;
@@ -558,19 +559,6 @@ let user_closed sock  msg =
   user_socks := List2.removeq sock !user_socks;
   ()
 
-(* Here, we clearly need a good html parser to remove tags, and to translate
-special characters. Avoid them in the meantime. *)
-let text_of_html html =
-  String2.convert false (fun buf state c ->
-    if state then
-      c <> '>'
-    else
-      if c = '<' then true else begin
-        Buffer.add_char buf c;
-        false
-      end
-) html
-
 let telnet_handler t event =
   match event with
     TcpServerSocket.CONNECTION (s, Unix.ADDR_INET (from_ip, from_port)) ->
@@ -581,7 +569,7 @@ let telnet_handler t event =
           "telnet connection"
           s in
         let telnet = {
-            telnet_auth = ref (has_empty_password admin_user);
+            telnet_auth = ref (has_empty_password (admin_user ()));
             telnet_iac = false;
             telnet_wait = 0;
             telnet_buffer = Buffer.create 100;
@@ -591,9 +579,10 @@ let telnet_handler t event =
             conn_output = (if !!term_ansi then ANSI else TEXT);
             conn_sortvd = NotSorted;
             conn_filter = (fun _ -> ());
-            conn_user = default_user;
+            conn_user = find_ui_user CommonUserDb.admin_user_name;
             conn_width = 80;
             conn_height = 0;
+            conn_info = Some (TELNET, (from_ip, from_port));
           } in
 	(match Ip_set.match_ip !allowed_ips_set from_ip with
 	| Some br -> 
@@ -603,6 +592,7 @@ let telnet_handler t event =
         TcpBufferedSocket.set_closer sock user_closed;
         user_socks := sock :: !user_socks;
 
+        TcpBufferedSocket.write_string sock iac_will_8bit;
         TcpBufferedSocket.write_string sock iac_will_naws;
 
         before_telnet_output o sock;
@@ -681,6 +671,7 @@ let extension_to_file_ext extension =
   | "jpeg"
   | "jpg" -> JPEG
   | "js" -> JAVASCRIPT
+  | "vob"
   | "mpe"
   | "mpeg"
   | "mpg" -> MPEG
@@ -819,14 +810,25 @@ let http_add_bin_stream_header r ext =
   add_reply_header r "Accept-Ranges" "bytes"
 
 let http_send_bin r buf filename =
-  let file_to_send = File.to_string filename in
-  let clen = String.length file_to_send in
-  let ext_pos = (String.rindex filename '.') + 1 in
-  let exten = (String.sub filename ext_pos ((String.length filename) - ext_pos)) in
-  if !verbose_msg_servers then
-    lprintf_nl "Extension found [%s] for file: [%s]" exten filename;
-  let ext = extension_to_file_ext exten in
-  http_add_bin_header r ext clen;
+  let file_to_send =
+    if theme_page_exists filename then
+      File.to_string (get_theme_page filename)
+    else
+      try
+        File.to_string filename
+      with _ ->
+        try
+          Hashtbl.find CommonPictures.files filename
+        with Not_found ->
+          try
+            if String.sub filename 0 4 = "flag" then
+              Hashtbl.find CommonPictures.files "flag_--.png"
+            else
+              raise Not_found
+          with _ -> raise Not_found
+  in
+  let ext = extension_to_file_ext (Filename2.last_extension2 filename) in
+  http_add_bin_header r ext (String.length file_to_send);
   Buffer.add_string buf file_to_send
 
 let http_error_no_gd img_type =
@@ -911,6 +913,8 @@ let send_preview r file fd size filename exten =
       else String.lowercase (String.sub exten 1 (len - 1)) in
   http_add_bin_stream_header r (extension_to_file_ext exten);
 
+  add_reply_header r "Content-Disposition"
+  (Printf.sprintf "inline;filename=\"%s\"" (Filename.basename filename)); 
   let s = String.create 200000 in
   set_max_output_buffer r.sock (String.length s);
   set_rtimeout r.sock 10000.;
@@ -945,7 +949,7 @@ let http_handler o t r =
 	 List.iter (fun (arg, value) -> Printf.bprintf b " %s %s" arg value) r.get_url.Url.args;
 	 if Buffer.contents b <> "" then Printf.sprintf "(%s)" (Buffer.contents b) else "");
 
-  let user = if r.options.login = "" then admin_user.CommonTypes.user_name else r.options.login in
+  let user = if r.options.login = "" then (admin_user ()).CommonTypes.user_name else r.options.login in
   if not (valid_password user r.options.passwd) then begin
       clear_page buf;
       http_file_type := HTM;
@@ -962,8 +966,10 @@ let http_handler o t r =
     begin
       let user = find_ui_user user  in
       let o = match user.ui_http_conn with
-          Some oo -> oo.conn_buf <- o.conn_buf; oo
-        | None -> let oo = { o with conn_user = user } in
+          Some oo -> oo.conn_buf <- o.conn_buf;
+            oo.conn_info <- Some (WEB, peer_addr t); oo
+        | None -> let oo = { o with conn_user = user;
+                                    conn_info = Some (WEB, peer_addr t)} in
             user.ui_http_conn <- Some oo; oo
       in
       try
@@ -1189,11 +1195,6 @@ let http_handler o t r =
                 http_send_bin r buf "tag.jpg"
             | true -> raise Not_found)
 
-        | "favicon.ico" ->
-            if !verbose_msg_servers then
-              lprintf_nl "favicon.ico request received by tracker";
-            http_send_bin r buf "favicon.ico"
-
         | "filter" ->
             html_open_page buf t r true;
             let b = Buffer.create 10000 in
@@ -1392,9 +1393,16 @@ let http_handler o t r =
 
               | [ "setoption", _ ; "option", name; "value", value ] ->
                   html_open_page buf t r true;
+                  let gui_type, ip, port =
+                    match o.conn_info with
+                    | None -> None, None, None
+                    | Some (gui_type, (ip, port)) -> Some gui_type, Some ip, Some port
+                  in
 		  if user2_is_admin o.conn_user.ui_user then
 		    begin
-                      CommonInteractive.set_fully_qualified_options name value;
+                      CommonInteractive.set_fully_qualified_options name value
+                        ~user:(Some o.conn_user.ui_user.CommonTypes.user_name)
+                        ~ip:ip ~port:port ~gui_type:gui_type ();
                       Buffer.add_string buf "Option value changed"
 		    end
 		  else
@@ -1493,26 +1501,7 @@ let http_handler o t r =
                 read_theme_page this_page else
               if !!html_mods then !!CommonMessages.download_html_js_mods0
               else !!CommonMessages.download_html_js_old)
-        | "porttest" ->
-	    html_open_page buf t r true;
-	    let age time =
-	      Date.time_to_string (BasicSocket.last_time () - time) "verbose" in
-	    networks_iter (fun n -> 
-	      let result =
-		match network_porttest_result n with
-		  PorttestNotAvailable -> None
-	        | PorttestNotStarted -> Some "porttest not started"
-	        | PorttestInProgress time ->
-		    Some (Printf.sprintf "porttest started %s ago" (age time))
-	        | PorttestResult (time, s) ->
-		    Some (Printf.sprintf "porttest finished %s ago, %s" (age time) s)
-	      in
-	      (match result with
-		None -> ()
-	      | Some result ->
-		    Printf.bprintf buf "%s:<br> %s<br>\n" n.network_name result));
-		    Printf.bprintf buf "<br><br><a href=\"porttest\">Reload</a>"
-        | _ -> raise Not_found
+        | s ->  http_send_bin r buf (String.lowercase s)
       with
       | Not_found ->
 	  let _, error_text_long, header = Http_server.error_page "404" "" ""
@@ -1536,7 +1525,7 @@ let http_handler o t r =
   in
   r.reply_content <- 
     if !http_file_type <> BIN && !!html_use_gzip then 
-      Autoconf.zlib__gzip_string s 
+      Zlib.gzip_string s 
     else s
 
 let http_options = {
@@ -1544,8 +1533,9 @@ let http_options = {
     conn_output = HTML;
     conn_sortvd = NotSorted;
     conn_filter = (fun _ -> ());
-    conn_user = default_user;
+    conn_user = find_ui_user CommonUserDb.admin_user_name;
     conn_width = 80; conn_height = 0;
+    conn_info = Some (WEB, (Ip.null, 0));
   }
 
 let create_http_handler () =
@@ -1554,6 +1544,7 @@ let create_http_handler () =
       port = !!http_port;
       requests = [];
       addrs = Ip_set.of_list !!allowed_ips;
+(* do not limit access to MLDonkey web interface by IP blocklist *)
       use_ip_block_list = false;
       base_ref = "";
       default = http_handler http_options;
