@@ -110,6 +110,16 @@ let connect_trackers file event f =
         | _ -> [],true,local_downloaded, left
   in
 
+  let args = ("no_peer_id", "1") :: ("compact", "1") :: args in
+  let args = if !!numwant > -1 then
+      ("numwant", string_of_int !!numwant) :: args else args
+  in
+  let args = if !!send_key then
+      ("key", Sha1.to_hexa !!client_uid) :: args else args
+  in
+   let args = if !!force_client_ip then
+      ("ip", Ip.to_string !!set_client_ip) :: args else args
+  in
   let args =
     ("info_hash", Sha1.direct_to_string file.file_id) ::
     ("peer_id", Sha1.direct_to_string !!client_uid) ::
@@ -117,18 +127,9 @@ let connect_trackers file event f =
     ("uploaded", Int64.to_string file.file_uploaded) ::
     ("downloaded", Int64.to_string downloaded) ::
     ("left", Int64.to_string left) ::
-    ("compact","1") ::
     args
   in
-  let args = if !!send_key then
-      ("key", Sha1.to_hexa !!client_uid) :: args else args
-  in
-  let args = if !!numwant > -1 then
-      ("numwant", string_of_int !!numwant) :: args else args
-  in
-  let args = if !!force_client_ip then
-      ("ip", Ip.to_string !!set_client_ip) :: args else args
-  in
+  
 
   let enabled_trackers =
     let enabled_trackers = List.filter (fun t -> tracker_is_enabled t) file.file_trackers in
@@ -140,15 +141,15 @@ let connect_trackers file event f =
       List.iter (fun t ->
 	match t.tracker_status with
       (* only re-enable after normal error *)
-	  Disabled _ -> t.tracker_status <- Enabled
+        | Disabled _ -> t.tracker_status <- Enabled
 	| _ -> ()) file.file_trackers;
-      let enabled_trackers = List.filter (fun t -> t.tracker_status = Enabled) file.file_trackers in
+      let enabled_trackers = List.filter (fun t -> tracker_is_enabled t) file.file_trackers in
       if enabled_trackers = [] && (file_state file) <> FilePaused then
 	begin
-	  file_pause (as_file file) CommonUserDb.admin_user;
+	  file_pause (as_file file) (CommonUserDb.admin_user ());
 	  lprintf_file_nl (as_file file) "Paused %s, no usable trackers left" (file_best_name (as_file file))
 	end;
-      file.file_trackers;
+      enabled_trackers
     end in
 
   List.iter (fun t ->
@@ -272,6 +273,8 @@ let disconnect_client c reason =
         try
 (*          List.iter (fun r -> CommonSwarming.free_range r) c.client_ranges; *)
           set_client_disconnected c reason;
+          c.client_session_downloaded <- 0L;
+          c.client_session_uploaded <- 0L;
           (try if c.client_good then count_seen c with _ -> ());
           (* this is not useful already done in the match
           (try close sock reason with _ -> ());   *)
@@ -459,17 +462,20 @@ let rec client_parse_header counter cc init_sent gconn sock
     let file = Hashtbl.find files_by_uid file_id in
     if !verbose_msg_clients then
       lprintf_file_nl (as_file file) "file found";
+    let ccc, cc_country_code = !cc in
     let c =
-      match !cc with
+      match ccc with
         None ->
-          let c = new_client file Sha1.null (TcpBufferedSocket.peer_addr sock) in
+          let c = new_client file Sha1.null (TcpBufferedSocket.peer_addr sock) cc_country_code in
           if !verbose_connect then lprintf_file_nl (as_file file) "Client %d: incoming connection" (client_num c);
-          cc := Some c;
+          cc := (Some c), cc_country_code;
           c
       | Some c ->
           (* Does it happen that this c was already used to connect successfully?
              If yes then this must happen: *)
           c.client_received_peer_id <- false;
+          if cc_country_code <> None && c.client_country_code = None then
+            c.client_country_code <- cc_country_code;
           c
           (* client could have had Sha1.null as peer_id/uid *)
           (* this is to be done, later
@@ -551,6 +557,16 @@ let rec client_parse_header counter cc init_sent gconn sock
         bt_handler TcpMessages.parsing (client_to_client c) c sock
     );
 
+    let b = TcpBufferedSocket.buf sock in
+(* The receive buffer is normally not empty now, lets parse the rest, most likely PeerID *)
+    if b.len <> 0 then
+      ignore (bt_handler TcpMessages.parsing (client_to_client c) c sock);
+
+(* Some newer clients send more opcodes in their handshake packet, lets parse them now.
+   Using "while b.len <> 0 do ... done" is not possible here because libtorrent clients
+   send unparsable five extra bytes after their PeerID which would result into a loop *)
+    if b.len <> 0 then
+      ignore (bt_handler TcpMessages.parsing (client_to_client c) c sock);
     ()
   with
       | Not_found ->
@@ -781,9 +797,10 @@ and get_from_client sock (c: client) =
 *)
 and client_to_client c sock msg =
   if !verbose_msg_clients then begin
+      let (ip,port) = (TcpBufferedSocket.peer_addr sock) in
       let (timeout, next) = get_rtimeout sock in
-      lprintf_nl "CLIENT %d: (%d, %d,%d) Received %s"
-        (client_num c)
+      lprintf_nl "CLIENT %d(%s:%d): (%d, %d,%d) Received %s"
+        (client_num c) (Ip.to_string ip) port
       (last_time ())
       (int_of_float timeout)
       (int_of_float next)
@@ -842,7 +859,7 @@ and client_to_client c sock msg =
             (*Update rate and amount of data received from client*)
             count_download c (new_downloaded -- old_downloaded);
             (* use len here with max_dr quickfix *)
-            Rate.update c.client_downloaded_rate  (float_of_int len);
+            Rate.update c.client_downloaded_rate ~amount:len;
             if !verbose_msg_clients then
               (match c.client_ranges_sent with
                   [] -> lprintf_file_nl (as_file file) "EMPTY Ranges !!!"
@@ -867,16 +884,6 @@ and client_to_client c sock msg =
               c.client_ranges_sent <- tail;
         end;
         get_from_client sock c;
-        if (List.length !current_uploaders < (!!max_bt_uploaders-1)) &&
-          (List.mem c (!current_uploaders)) == false && c.client_interested then
-          begin
-            (*we are probably an optimistic uploaders for this client
-              don't miss the oportunity if we can
-            *)
-            current_uploaders := c::(!current_uploaders);
-            c.client_sent_choke <- false;
-	    start_upload c
-          end;
 
         (* Check if the client is still interesting for us... *)
         check_if_interesting file c
@@ -889,24 +896,8 @@ and client_to_client c sock msg =
           let brand, release = parse_software p in
           c.client_brand <- brand;
           c.client_release <- release;
-(* TODO : enable it
-      c.client_release <- (parse_release p c.client_brand);
- *)
-
-          if (List.length !current_uploaders < (!!max_bt_uploaders-1)) &&
-            (List.mem c (!current_uploaders)) == false then
-            begin
-            (*we are probably an optimistic uploader for this client
-              don't miss the opportunity if we can *)
-              current_uploaders := c::(!current_uploaders);
-              c.client_sent_choke <- false;
-	      start_upload c
-            end
-          else
-            begin
-              send_client c Choke;
-              c.client_sent_choke <- true;
-            end
+          send_client c Choke;
+          c.client_sent_choke <- true;
         end
       else
         disconnect_client c Closed_by_user
@@ -1045,7 +1036,7 @@ and client_to_client c sock msg =
             raise Exit
           end;
 
-        if !CommonUploads.has_upload = 0 then
+        if !CommonGlobals.has_upload = 0 then
           begin
             if client_has_a_slot (as_client c) then
               begin
@@ -1096,12 +1087,14 @@ be put in a fifo and dequeued according to
 let connect_client c =
   if can_open_connection connection_manager &&
     (let (ip,port) = c.client_host in
-     match !Ip.banned ip with
+     match !Ip.banned (ip, c.client_country_code) with
        None -> true
      | Some reason ->
          if !verbose_connect then
-           lprintf_nl "%s:%d blocked: %s"
-             (Ip.to_string ip) port reason;
+           lprintf_nl "%s:%d (%s), blocked: %s"
+             (Ip.to_string ip) port
+             (fst (Geoip.get_country_code_name c.client_country_code))
+             reason;
          false)
   then
   match c.client_sock with
@@ -1161,7 +1154,7 @@ let connect_client c =
                     be parsed
                   *)
                   set_bt_sock sock !verbose_msg_clients
-                    (BTHeader (client_parse_header !counter (ref (Some c)) true))
+                    (BTHeader (client_parse_header !counter (ref ((Some c), c.client_country_code)) true))
                 end
             with e ->
                 lprintf_nl "Exception %s while connecting to client"
@@ -1193,6 +1186,7 @@ let listen () =
                 to connect to us
               *)
               let ip = (Ip.of_inet_addr from_ip) in
+              let cc = Geoip.get_country_code_option ip in
               if !verbose_sources > 1 then lprintf_nl "CONNECTION RECEIVED FROM %s"
                 (Ip.to_string (Ip.of_inet_addr from_ip))
               ;
@@ -1200,12 +1194,14 @@ let listen () =
                 to bypass the max_connection parameter
               *)
               if can_open_connection connection_manager &&
-                (match !Ip.banned ip with
+                (match !Ip.banned (ip, cc) with
                    None -> true
                  | Some reason ->
                      if !verbose_connect then
-                       lprintf_nl "%s:%d blocked: %s"
-                         (Ip.to_string ip) from_port reason;
+                       lprintf_nl "%s:%d (%s) blocked: %s"
+                         (Ip.to_string ip) from_port
+                         (fst (Geoip.get_country_code_name cc))
+                         reason;
                      false)
               then
                 begin
@@ -1225,9 +1221,9 @@ let listen () =
                   TcpBufferedSocket.set_read_controler sock download_control;
                   TcpBufferedSocket.set_write_controler sock upload_control;
 
-                  let c = ref None in
+                  let c = ref (None, cc) in
                   TcpBufferedSocket.set_closer sock (fun _ r ->
-                      match !c with
+                      match fst !c with
                         Some c ->  begin
                             match c.client_sock with
                             | Connection s when s == sock ->
@@ -1282,7 +1278,7 @@ let resume_clients file =
       try
         match c.client_sock with
         | Connection sock -> ()
-            (*i think this one is not realy usefull for debugging
+            (*i think this one is not really usefull for debugging
               lprintf_nl "[BT]: RESUME: Client is already connected"; *)
         | _ ->
             (try
@@ -1350,12 +1346,26 @@ let get_sources_from_tracker file =
     match v with
       Dictionary list ->
         List.iter (fun (key,value) ->
+            (match (key, value) with
+            | String "failure reason", _ -> ()
+            | _ -> (match t.tracker_status with
+                   | Disabled_failure (i, _) ->
+                        lprintf_file_nl (as_file file) "Received good message from Tracker %s in file: %s after %d bad attempts"
+                          t.tracker_url file.file_name i
+                   | _ -> ());
+                   (* Received good message from tracker after failures, re-enable tracker *)
+                   t.tracker_status <- Enabled);
+
             match (key, value) with
             | String "failure reason", String failure ->
-                (* On failure, disable the tracker and forbid re-enabling *)
-		t.tracker_status <- Disabled_failure (intern failure);
-                lprintf_file_nl (as_file file) "Failure from Tracker %s in file: %s Reason: %s\nBT: Tracker %s disabled for failure"
-                  t.tracker_url file.file_name (Charset.to_utf8 failure) t.tracker_url
+                (* On failure, disable the tracker, count the attempts and forbid re-enabling *)
+		t.tracker_status <- (match t.tracker_status with
+                  | Disabled_failure (i,_) -> Disabled_failure (i + 1, intern failure)
+                  | _ -> Disabled_failure (1, intern failure));
+                lprintf_file_nl (as_file file) "Failure no. %d%s from Tracker %s in file: %s Reason: %s"
+                  (match t.tracker_status with | Disabled_failure (i,_) -> i | _ -> 1)
+                  (if !!tracker_retries = 0 then "" else Printf.sprintf "/%d" !!tracker_retries)
+                  t.tracker_url file.file_name (Charset.to_utf8 failure)
             | String "warning message", String warning ->
                 lprintf_file_nl (as_file file) "Warning from Tracker %s in file: %s Reason: %s" t.tracker_url file.file_name warning
             | String "interval", Int n ->
@@ -1416,11 +1426,12 @@ let get_sources_from_tracker file =
                         ) list;
 
                         (* Only record valid clients *)
+                        let cc = Geoip.get_country_code_option !peer_ip in
                         if !peer_id != Sha1.null &&
                           !peer_id <> !!client_uid &&
                           !peer_ip != Ip.null &&
                           !port <> 0 &&
-                          (match !Ip.banned !peer_ip with
+                          (match !Ip.banned (!peer_ip, cc) with
                               None -> true
                             | Some reason ->
                                 if !verbose_connect then
@@ -1428,10 +1439,10 @@ let get_sources_from_tracker file =
                                     (Ip.to_string !peer_ip) !port reason;
                                 false)
                         then
-                          let _ = new_client file !peer_id (!peer_ip,!port)
-                          in
-                          if !verbose_sources > 1 then lprintf_file_nl (as_file file) "Received %s:%d" (Ip.to_string !peer_ip)
-                          !port;
+                          ignore (new_client file !peer_id (!peer_ip,!port) cc);
+                          if !verbose_sources > 1 then
+                            lprintf_file_nl (as_file file) "Received %s:%d"
+                              (Ip.to_string !peer_ip) !port;
                           ()
                     | _ -> assert false
                 ) list
@@ -1442,7 +1453,7 @@ let get_sources_from_tracker file =
                         get_uint8 s (pos+2),get_uint8 s (pos+3))
                     and port = get_int16 s (pos+4)
                     in
-                    ignore( new_client file Sha1.null (ip,port));
+                    ignore (new_client file Sha1.null (ip,port) None);
                     t.tracker_last_clients_num <- t.tracker_last_clients_num + 1;
 
                     iter_comp s (pos+6) l
@@ -1517,7 +1528,8 @@ let rec iter_upload sock c =
 (*          lprintf "Unix32.read: offset %Ld len %d\n" offset len; *)
           Unix32.read (file_fd file) offset upload_buffer 0 len;
           (* update upload rate from len bytes *)
-          Rate.update c.client_upload_rate  (float_of_int len);
+          Rate.update c.client_upload_rate  ~amount:len;
+          Rate.update c.client_downloaded_rate;
           file.file_uploaded <- file.file_uploaded ++ (Int64.of_int len);
           let _ =
             (* update stats *)
@@ -1557,7 +1569,7 @@ let client_can_upload c allowed =
       | _ :: tail ->
     let new_allowed_to_write =
       c.client_allowed_to_write ++ (Int64.of_int allowed) in
-      if allowed > 0 && CommonUploads.can_write_len sock
+      if allowed > 0 && can_write_len sock
             (Int64.to_int new_allowed_to_write)
       then begin
         CommonUploads.consume_bandwidth allowed;
@@ -1604,6 +1616,7 @@ let _ =
        (*When a file is paused we consider it is stopped*)
       file_stop file
   );
+  file_ops.op_file_queue <- file_ops.op_file_pause;
   client_ops.op_client_enter_upload_queue <- (fun c ->
       if !verbose_msg_clients then
         lprintf_nl "Client %d: client_enter_upload_queue" (client_num c);
