@@ -59,7 +59,7 @@ torrents/: for BitTorrent
 let chunk_size = Int64.of_int (256 * 1024)
 
 let decode_torrent s =
-  if !verbose_msg_servers then lprintf_nl "[BT]: .torrent file loaded";
+  if !verbose_msg_servers then lprintf_nl "[BT] .torrent file loaded";
 (*            lprintf "Loaded: %s\n" (String.escaped s); *)
   let v = Bencode.decode s in
 (*            lprintf "Decoded file: %s\n" (Bencode.print v);  *)
@@ -70,7 +70,7 @@ let decode_torrent s =
   let file_info = ref (List []) in
   let file_name = ref "" in
   let file_torrent_filename = ref "" in
-  let file_name_utf8 = ref "" in
+  let file_name_utf8 = ref None in
   let file_piece_size = ref zero in
   let file_pieces = ref "" in
   let file_comment = ref "" in
@@ -79,7 +79,6 @@ let decode_torrent s =
   let file_modified_by = ref "" in
   let file_encoding = ref "" in
   let file_ed2k_hash = ref "" in
-  let file_nodes = ref "" in
   let file_is_private = ref zero in
   let file_aps = ref (List []) in
   let file_dht_backup_enable = ref zero in
@@ -95,28 +94,38 @@ let decode_torrent s =
             let current_length = ref zero in
             let length_set = ref false in
 
-            List.iter (fun (key, value) ->
-                match key, value with
-                  String "path", List path ->
-                    current_file :=
+            let path_list_to_string l =
                     Filepath.path_to_string '/'
-                      (List.map (fun v ->
+                (
+                  List.map (fun v ->
                           match v with
                             String s -> s
                           | _ -> assert false
-                      ) path);
+                  ) l
+                )
+            in
+
+            List.iter (fun (key, value) ->
+                match key, value with
+                  String "path", List path ->
+                    if !current_file = "" then begin
+                      current_file := path_list_to_string path;
+                      if !verbose_msg_servers then
+                        lprintf_nl "[BT] Parsed a new path: [%s]" !current_file
+                    end   
+                | String "path.utf-8", List path_utf8 -> 
+                    current_file := path_list_to_string path_utf8;
                     if !verbose_msg_servers then
-                      lprintf_nl "[BT]: New file received :%s" !current_file
-                | String "path.utf-8", String path_utf8 -> ()
+                      lprintf_nl "[BT] Parsed path.utf-8: [%s]" !current_file
                 | String "length", Int n ->
                     length := !length ++ n;
                     current_length := n;
                     length_set := true
 
                 | String key, _ ->
-                    if !verbose_msg_servers then lprintf_nl "[BT]: other field [%s] with value [%s] in files" key (Bencode.print value)
+                    if !verbose_msg_servers then lprintf_nl "[BT] other field [%s] with value [%s] in files" key (Bencode.print value)
                 | _ ->
-                    lprintf_nl "[BT]: other field in files"
+                    lprintf_nl "[BT] other field in files"
             ) list;
 
             assert (!length_set);
@@ -135,7 +144,7 @@ let decode_torrent s =
             match key, value with
               String "announce", String tracker_url ->
                if !verbose_msg_servers then
-                 lprintf_nl "[BT]: New tracker added :%s" tracker_url;
+                 lprintf_nl "[BT] New tracker added :%s" tracker_url;
                 announce := tracker_url
             | String "announce-list", List list ->
                 List.iter (fun url_list ->
@@ -147,10 +156,10 @@ let decode_torrent s =
                             | String next_url ->
                                 next_urls := next_url :: !next_urls;
                                 if !verbose_msg_servers then
-                                  lprintf_nl "[BT]: New tracker received :%s" next_url
+                                  lprintf_nl "[BT] New tracker received :%s" next_url
                             | _ ->
                                 if !verbose_msg_servers then
-                                  lprintf_nl "[BT]: error while decoding announce list"
+                                  lprintf_nl "[BT] error while decoding announce list"
                         ) next_url_list;
                         if List.length !next_urls > 1 then begin
                             next_urls := List2.shuffle !next_urls;
@@ -159,12 +168,12 @@ let decode_torrent s =
                         else
                           announce_list := List.hd !next_urls :: !announce_list
                     | _ ->
-                        lprintf_nl "[BT]: unknown field in announce list"
+                        lprintf_nl "[BT] unknown field in announce list"
                     ) list;
                     announce_list := List.rev !announce_list;
                     if !verbose_msg_servers then
                       List.iter (fun url ->
-                        lprintf_nl "[BT]: New tracker added :%s" url
+                        lprintf_nl "[BT] New tracker added :%s" url
                         ) !announce_list
             | String "info", ((Dictionary list) as info) ->
 
@@ -195,7 +204,7 @@ let decode_torrent s =
                       file_created_by := !file_created_by ^ " @ " ^ publisher_url
 
                     | String "name.utf-8", String name_utf8 ->
-                        file_name_utf8 := name_utf8
+                        file_name_utf8 := Some name_utf8
 
                     | String "publisher.utf-8", String publisher_utf8 -> ()
                     | String "publisher-url.utf-8", String publisher_url_utf8 -> ()
@@ -205,12 +214,12 @@ let decode_torrent s =
                         file_is_private := n;
                         if !verbose_msg_servers &&
                           Int64.to_int !file_is_private = 1 then
-                            lprintf_nl "[BT]: torrent is private"
+                            lprintf_nl "[BT] torrent is private"
                     | String key, _ ->
                         if !verbose_msg_servers then
-                          lprintf_nl "[BT]: found other field [%s] with value [%s] in info" key (Bencode.print value)
+                          lprintf_nl "[BT] found other field [%s] with value [%s] in info" key (Bencode.print value)
                     | _ ->
-                        lprintf_nl "[BT]: other field in info"
+                        lprintf_nl "[BT] other field in info"
                 ) list
 
             | String "comment", String comment
@@ -247,57 +256,59 @@ let decode_torrent s =
                         file_dht_backup_enable := n;
                         if !verbose_msg_servers &&
                           Int64.to_int !file_dht_backup_enable = 1 then
-                            lprintf_nl "[BT]: azureus properties : Torrent has dht backup"
+                            lprintf_nl "[BT] azureus properties : Torrent has dht backup"
                     | String key, _ ->
                         if !verbose_msg_servers then
-                          lprintf_nl "[BT]: found other field [%s] with value [%s] in azureus properties" key (Bencode.print value)
+                          lprintf_nl "[BT] found other field [%s] with value [%s] in azureus properties" key (Bencode.print value)
                     | _ ->
-                        lprintf_nl "[BT]: other field in azureus properties"
+                        lprintf_nl "[BT] other field in azureus properties"
                 ) list
             | String key, _ ->
-                if !verbose_msg_servers then lprintf_nl "[BT]: found other field [%s] with value [%s] after info" key (Bencode.print value)
+                if !verbose_msg_servers then lprintf_nl "[BT] found other field [%s] with value [%s] after info" key (Bencode.print value)
             | _ ->
-                lprintf_nl "[BT]: other field after info"
+                lprintf_nl "[BT] other field after info"
         ) list
     | _ -> assert false
   end;
 
+  let real_file_name =
+    match !file_name_utf8 with
+    | None -> Charset.safe_convert !file_encoding !file_name
+    | Some name -> name
+  in
   assert (!announce <> "");
-  assert (!file_name <> "");
+  assert (real_file_name <> "");
   assert (!file_piece_size <> zero);
   assert (!file_pieces <> "");
-
   assert (!file_info = Bencode.decode (Bencode.encode !file_info));
 
   let file_id = Sha1.string (Bencode.encode !file_info) in
-  let npieces =
-    1+ Int64.to_int ((!length -- one) // !file_piece_size)
-  in
-(*            lprintf "npieces %d length %Ld piece %Ld %d\n"
-              npieces !length !file_piece_size (String.length !file_pieces); *)
+  let npieces = 1 + Int64.to_int ((!length -- one) // !file_piece_size) in
   let pieces = Array.init npieces (fun i ->
         let s = String.sub !file_pieces (i*20) 20 in
         Sha1.direct_of_string s
     ) in
 
-(*  if !file_files <> [] && not (String2.check_suffix !file_name ".torrent") then
-    file_name := !file_name ^ ".torrent";*)
-  file_files := List.rev !file_files;
+  (match List.length !file_files with
+  | 0 -> ()
+  | 1 -> file_name := (fst (List.hd !file_files));
+         file_files := []
+  | _ -> file_files := List.rev !file_files);
 
   file_id, {
-    torrent_name = !file_name;
+    torrent_name = real_file_name;
     torrent_filename = !file_torrent_filename;
-    torrent_name_utf8 = !file_name_utf8;
+    torrent_name_utf8 = real_file_name;
     torrent_length = !length;
     torrent_announce = !announce;
     torrent_announce_list = !announce_list;
     torrent_piece_size = !file_piece_size;
     torrent_files = !file_files;
     torrent_pieces = pieces;
-    torrent_comment = !file_comment;
-    torrent_created_by = !file_created_by;
+    torrent_comment = Charset.safe_convert !file_encoding !file_comment;
+    torrent_created_by = Charset.safe_convert !file_encoding !file_created_by;
     torrent_creation_date = !file_creation_date;
-    torrent_modified_by = !file_modified_by;
+    torrent_modified_by = Charset.safe_convert !file_encoding !file_modified_by;
     torrent_encoding = !file_encoding;
     torrent_private = !file_is_private;
 
@@ -317,9 +328,9 @@ let encode_torrent torrent =
 
   let encode_file (filename, size) =
     Dictionary [
+      String "length", Int size;
       String "path", List (List.map
           (fun s -> String s)(Filepath.string_to_path '/' filename));
-      String "length", Int size;
     ]
   in
 
@@ -348,11 +359,11 @@ let encode_torrent torrent =
   file_id,
   Dictionary [
     String "announce", String torrent.torrent_announce;
-    String "info", info;
     String "comment", String torrent.torrent_comment;
     String "created by", String torrent.torrent_created_by;
     String "creation date", Int torrent.torrent_creation_date;
     String "encoding", String torrent.torrent_encoding;
+    String "info", info;
     String "modified-by", String torrent.torrent_modified_by;
 (*
     String "nodes", String torrent.torrent_nodes;
@@ -378,19 +389,19 @@ let make_torrent announce filename comment is_private =
               if Unix2.is_directory fullname then
                 iter_directory list basename
               else
-                (basename, Unix32.getsize fullname false) :: list
+                (basename, Unix32.getsize fullname) :: list
             in
             iter_files left dirname tail
       in
       let files = iter_directory [] "" in
-      let t = Unix32.create_multifile filename Unix32.ro_flag 0o666 files in
+      let t = Unix32.create_multifile filename false files in
       files, t
     else
       [], Unix32.create_ro filename
   in
 
   Unix32.flush_fd t;
-  let length = Unix32.getsize64 t false in
+  let length = Unix32.getsize64 t in
   let npieces = 1+ Int64.to_int ((length -- one) // chunk_size) in
   let pieces = Array.create npieces Sha1.null in
   for i = 0 to npieces - 1 do
