@@ -137,14 +137,14 @@ let reset_tags () =
   let emule_compatoptions = D.emule_compatoptions m in
   client_to_client_tags :=
   [
-    string_tag (Field_UNKNOWN "name") (local_login ());
-    int_tag (Field_UNKNOWN "port") !!donkey_port;
-    int_tag (Field_UNKNOWN "version") protocol_version;
-    int_tag (Field_UNKNOWN "emule_udpports") (!!donkey_port+4);
-    int_tag (Field_UNKNOWN "emule_version") m.emule_version;
-    int64_tag (Field_UNKNOWN "emule_miscoptions1") emule_miscoptions1;
-    int64_tag (Field_UNKNOWN "emule_miscoptions2") emule_miscoptions2;
-    int_tag (Field_UNKNOWN "emule_compatoptions") emule_compatoptions;
+    string_tag (Field_KNOWN "name") (local_login ());
+    int_tag (Field_KNOWN "port") !!donkey_port;
+    int_tag (Field_KNOWN "version") protocol_version;
+    int_tag (Field_KNOWN "emule_udpports") (!!donkey_port+4);
+    int_tag (Field_KNOWN "emule_version") m.emule_version;
+    int64_tag (Field_KNOWN "emule_miscoptions1") emule_miscoptions1;
+    int64_tag (Field_KNOWN "emule_miscoptions2") emule_miscoptions2;
+    int_tag (Field_KNOWN "emule_compatoptions") emule_compatoptions;
   ];
 
 (* server capabilities *)
@@ -154,49 +154,49 @@ let reset_tags () =
   extended := !extended lor 0x04; (* support of auxport *)
   extended := !extended lor 0x08; (* newtags *)
 (*extended := !extended lor 0x10; (* unicode *) *)
-(*extended := !extended lor 0x100; (* files > 4GB *) *)
+  extended := !extended lor 0x100; (* files > 4GB *)
 (*extended := !extended lor 0x200; (* support crypt *) *)
 (*extended := !extended lor 0x400; (* request crypt *) *)
 (*extended := !extended lor 0x800; (* require crypt *) *)
 
   client_to_server_tags :=
   [
-    string_tag (Field_UNKNOWN "name") (local_login ());
-    int_tag (Field_UNKNOWN "version") protocol_version;
-    int_tag (Field_UNKNOWN "extended") !extended;
-    int_tag (Field_UNKNOWN "emule_version") m.emule_version;
+    string_tag (Field_KNOWN "name") (local_login ());
+    int_tag (Field_KNOWN "version") protocol_version;
+    int_tag (Field_KNOWN "extended") !extended;
+    int_tag (Field_KNOWN "emule_version") m.emule_version;
   ];
 
   client_to_server_reply_tags :=
   [
-    string_tag (Field_UNKNOWN "name") (local_login ());
-    int_tag (Field_UNKNOWN "version") protocol_version;
-    int_tag (Field_UNKNOWN "emule_udpports") (!!donkey_port+4);
-    int64_tag (Field_UNKNOWN "emule_miscoptions1") emule_miscoptions1;
-    int64_tag (Field_UNKNOWN "emule_miscoptions2") emule_miscoptions2;
-    int_tag (Field_UNKNOWN "emule_version") m.emule_version;
+    string_tag (Field_KNOWN "name") (local_login ());
+    int_tag (Field_KNOWN "version") protocol_version;
+    int_tag (Field_KNOWN "emule_udpports") (!!donkey_port+4);
+    int64_tag (Field_KNOWN "emule_miscoptions1") emule_miscoptions1;
+    int64_tag (Field_KNOWN "emule_miscoptions2") emule_miscoptions2;
+    int_tag (Field_KNOWN "emule_version") m.emule_version;
   ];
 
   emule_info.DonkeyProtoClient.EmuleClientInfo.tags <- [
-    int_tag (Field_UNKNOWN "compression") m.emule_compression;
-    int_tag (Field_UNKNOWN "udpver") m.emule_udpver;
-    int_tag (Field_UNKNOWN "udpport") (!!donkey_port+4);
-    int_tag (Field_UNKNOWN "sourceexchange") m.emule_sourceexchange;
-    int_tag (Field_UNKNOWN "comments") m.emule_comments;
-    int_tag (Field_UNKNOWN "compatibleclient") !DonkeyProtoClient.compatibleclient; 
-    int_tag (Field_UNKNOWN "extendedrequest") m.emule_extendedrequest;
-    int_tag (Field_UNKNOWN "features") m.emule_features;
+    int_tag (Field_KNOWN "compression") m.emule_compression;
+    int_tag (Field_KNOWN "udpver") m.emule_udpver;
+    int_tag (Field_KNOWN "udpport") (!!donkey_port+4);
+    int_tag (Field_KNOWN "sourceexchange") m.emule_sourceexchange;
+    int_tag (Field_KNOWN "comments") m.emule_comments;
+    int_tag (Field_KNOWN "compatibleclient") !DonkeyProtoClient.compatibleclient; 
+    int_tag (Field_KNOWN "extendedrequest") m.emule_extendedrequest;
+    int_tag (Field_KNOWN "features") m.emule_features;
     
   ];
   overnet_connect_tags :=
   [
-    string_tag (Field_UNKNOWN "name") (local_login ());
-    int_tag (Field_UNKNOWN "version") !!DonkeyProtoOvernet.overnet_protocol_connect_version; 
+    string_tag (Field_KNOWN "name") (local_login ());
+    int_tag (Field_KNOWN "version") !!DonkeyProtoOvernet.overnet_protocol_connect_version; 
   ];
   overnet_connectreply_tags :=
   [
-    string_tag (Field_UNKNOWN "name") (local_login ());
-    int_tag (Field_UNKNOWN "version") !!DonkeyProtoOvernet.overnet_protocol_connectreply_version; 
+    string_tag (Field_KNOWN "name") (local_login ());
+    int_tag (Field_KNOWN "version") !!DonkeyProtoOvernet.overnet_protocol_connectreply_version; 
   ]
   
 let enable () =
@@ -263,12 +263,15 @@ let enable () =
 be useful when users want to share files that they had already previously
   shared *)
           let key = (file.sh_name, file.sh_size, file.sh_mtime) in
+          (try
           if Unix32.file_exists file.sh_name &&
             not (Hashtbl.mem shared_files_info key) 
             then begin
               Hashtbl.add shared_files_info key file;
               list := file :: !list
             end
+          with e ->
+            lprintf_nl "ignoring share: %s" (Printexc2.to_string e))
       ) !!known_shared_files;
       known_shared_files =:= !list;
 
@@ -278,7 +281,7 @@ be useful when users want to share files that they had already previously
           let sock = TcpServerSocket.create 
               "donkey client server"
               (Ip.to_inet_addr !!client_bind_addr)
-            !!donkey_port (client_connection_handler false) in
+            !!donkey_port ~backlog:!!max_upload_slots (client_connection_handler false) in
           
           TcpServerSocket.set_accept_controler sock connections_controler;
           listen_sock := Some sock;
@@ -368,6 +371,11 @@ let rec update_options () =
       if !!upload_timeout = 60. then
         upload_timeout =:= 600.;
       update 3
+  | 3 ->
+      propagate_sources =:= false;
+      update_server_list_server =:= false;
+      upload_full_chunks =:= true;
+      update 4
   | _ -> ()
 
 
@@ -405,7 +413,10 @@ let _ =
     !kademlia_port_info, "kademlia_port UDP";
     ]);
   network.op_network_porttest_result <-
-    (fun _ -> !DonkeyInteractive.porttest_result);
+    (fun _ -> match !DonkeyInteractive.porttest_result with
+        | PorttestResult (time, s) ->
+            PorttestResult (time, (String2.dehtmlize s))
+        | _ -> !DonkeyInteractive.porttest_result);
   CommonInteractive.register_gui_options_panel "eDonkey" 
     gui_donkey_options_panel;
   CommonInteractive.register_gui_options_panel "Overnet" 
