@@ -30,8 +30,9 @@ let max_xs_packets = define_expert_option donkey_section ["max_xs_packets"]
     int_option 30
 
 let donkey_port = define_option donkey_section ["port"]
+  ~restart: true
   "The port used for connection by other donkey clients."
-    int_option (2000 + Random.int 20000)
+    port_option (2000 + Random.int 20000)
 
 let check_client_connections_delay = define_expert_option donkey_section ["check_client_connections_delay"]
   "Delay used to request file sources"
@@ -45,6 +46,10 @@ let max_connected_servers = define_option donkey_section ["max_connected_servers
   "The number of servers you want to stay connected to"
     int_option 3
 
+let max_allowed_connected_servers () =
+  min (int_of_string ((strings_of_option max_connected_servers).option_default))
+      !!max_connected_servers
+
 let reliable_sources = define_option donkey_section ["reliable_sources"]
   "Should mldonkey try to detect sources responsible for corruption and ban them, currently disabled"
     bool_option true
@@ -53,13 +58,16 @@ let ban_identity_thieves = define_option donkey_section ["ban_identity_thieves"]
   "Should mldonkey try to detect sources masquerading as others and ban them"
     bool_option true
 
-let max_allowed_connected_servers () =
-  BasicSocket.mini 5 !!max_connected_servers
-
 let server_black_list = define_option donkey_section ["server_black_list"]
-  "A list of server IP to remove from server list.
+  "A list of server IP to remove from server list. Can contain single IPs, CIDR ranges, or begin-end ranges.
   Servers on this list can't be added, and will eventually be removed"
-    (list_option Ip.option) []
+    CommonOptions.ip_range_list_option []
+
+let server_black_list_set = ref Ip_set.BL_Empty
+
+let () =
+  option_hook server_black_list (fun _ ->
+    server_black_list_set := Ip_set.of_list !!server_black_list)
 
 let force_high_id = define_option donkey_section ["force_high_id"]
   "immediately close connection to servers that don't grant a High ID"
@@ -72,7 +80,7 @@ let force_client_high_id = define_option donkey_section ["force_client_high_id"]
 let update_server_list_server = define_option donkey_section ["update_server_list_server"]
   "Set this option to false if you don't want to
   receive new servers from servers"
-    bool_option true
+    bool_option false
 
 let update_server_list_server_met = define_option donkey_section ["update_server_list_server_met"]
   "Set this option to false if you don't want to
@@ -100,8 +108,7 @@ let max_walker_servers = define_expert_option donkey_section ["max_walker_server
     int_option 1
 
 let walker_server_lifetime = define_expert_option donkey_section ["walker_server_lifetime"]
-  "The maximal delay a connection with a server should last when walking
-  through the list (should be greater than become_master_delay)"
+  "The maximal delay a connection with a server should last when walking through the list"
     int_option 300
 
 let log_clients_on_console = define_expert_option donkey_section ["log_clients_on_console"]
@@ -109,8 +116,10 @@ let log_clients_on_console = define_expert_option donkey_section ["log_clients_o
     bool_option false
 
 let propagate_sources = define_expert_option donkey_section ["propagate_sources"]
-  "Allow mldonkey to propagate your sources to other donkey clients"
-    bool_option true
+  "Allow mldonkey to propagate your sources to other mldonkey clients.
+   This function is superseeded by eMule-style source exchange,
+   this option is outdated"
+    bool_option false
 
 let max_sources_per_file = define_option donkey_section ["max_sources_per_file"]
   "Maximal number of sources for each file"
@@ -168,6 +177,16 @@ let upload_lifetime = define_expert_option donkey_section ["upload_lifetime"]
   "How long a downloading client can stay in my upload queue (in minutes >5)"
     int_option 90
 
+let upload_full_chunks = define_expert_option donkey_section ["upload_full_chunks"]
+  "If true, each client is allowed to receive one chunk, this setting overrides upload_lifetime"
+    bool_option true
+
+let upload_complete_chunks = define_expert_option donkey_section ["upload_complete_chunks"]
+  "If true, each client is allowed to complete only one chunk, independent, if it is empty or
+   partial. this setting overrides upload_full_chunks and dynamic_upload_lifetime,
+   but is, as a failsafe, limited by upload_lifetime (should be set reasonable high)"
+    bool_option false
+
 let dynamic_upload_lifetime = define_expert_option donkey_section ["dynamic_upload_lifetime"]
   "Each client upload lifetime depends on download-upload ratio"
     bool_option false
@@ -194,6 +213,7 @@ let max_server_age = define_expert_option donkey_section ["max_server_age"]
     int_option 2
 
 let remove_old_servers_delay = define_expert_option donkey_section ["remove_old_servers_delay"]
+  ~restart: true
   "How often should remove old donkey servers (see max_server_age) be called
   (in seconds, 0 to disable)"
     float_option 900.
@@ -244,9 +264,13 @@ let good_client_rank = define_expert_option donkey_section ["good_client_rank"]
     int_option 500
 
 let min_users_on_server = define_option donkey_section ["min_users_on_server"]
-  "min connected users for each server"
+  "disconnect if server users is smaller"
     int_option 0
 
+let max_published_files = define_option donkey_section ["max_published_files"]
+  "maximum number of files published to servers per minute, eMule default 200"
+    int_option 200
+
 let login = define_option donkey_section ["login"]
   "login of client on eDonkey network (nothing default to global one)"
     string_option ""
@@ -257,16 +281,14 @@ let overnet_section = file_section donkey_ini [ overnet_options_section_name ]
     "Overnet options"
 
 let overnet_port = define_option overnet_section [overnet_options_section_name; "port"]
+  ~restart: true
   "port for overnet"
-    int_option (2000 + Random.int 20000)
-
-let become_master_delay = define_expert_option donkey_section ["become_master_delay"]
-  "(only for development tests)"
-    int_option 120
+    port_option (2000 + Random.int 20000)
 
 let options_version = define_expert_option donkey_section ["options_version"]
+  ~internal: true
   "(internal option)"
-    int_option 3
+    int_option 4
 
 let gui_donkey_options_panel =
   [
