@@ -138,7 +138,7 @@ let set_file_size file size =
       file_must_update (as_file file);
     end
 
-let new_file file_id file_name file_size =
+let new_file file_id file_name file_size user =
   let file_temp = Filename.concat !!temp_directory
       (Printf.sprintf "FileTP-%s" (Md4.to_string file_id)) in
   let t = Unix32.create_rw file_temp in
@@ -152,6 +152,8 @@ let new_file file_id file_name file_size =
       file_nconnected_clients = 0;
     } and file_impl =  {
       dummy_file_impl with
+      impl_file_owner = user;
+      impl_file_group = user.user_default_group;
       impl_file_fd = Some t;
       impl_file_size = zero;
       impl_file_downloaded = zero;
@@ -168,14 +170,22 @@ let new_file file_id file_name file_size =
 (*      lprintf "ADD FILE TO DOWNLOAD LIST\n"; *)
   file
 
-let new_file file_id file_name file_size =
+let new_file file_id file_name file_size users =
   try
     Hashtbl.find files_by_uid file_id
   with _ ->
-      let file = new_file file_id file_name file_size  in
+      let file = new_file file_id file_name file_size users in
       Hashtbl.add files_by_uid file_id file;
       file
 
+let check_client_country_code c =
+  if Geoip.active () then
+    match c.client_country_code with
+    | None ->
+        c.client_country_code <-
+          Geoip.get_country_code_option (Ip.from_name c.client_hostname)
+    | _ -> ()
+
 let new_client proto hostname port referer =
   let key = (hostname,port) in
   try
@@ -190,7 +200,9 @@ let new_client proto hostname port referer =
           client_hostname = hostname;
           client_referer = referer;
           client_port = port;
-	  client_downloaded = zero;
+          client_country_code = None;
+	  client_total_downloaded = zero;
+	  client_session_downloaded = zero;
           client_reconnect = false;
           client_in_queues = [];
           client_connected_for = None;
@@ -204,6 +216,7 @@ let new_client proto hostname port referer =
 	  impl_client_upload = None;
         } in
       new_client impl;
+      check_client_country_code c;
       Hashtbl.add clients_by_uid key c;
       c
     
