@@ -112,10 +112,12 @@ let string_of_client_addr c = c.client_hostname
 
 let _ =
   client_ops.op_client_info <- (fun c ->
+      check_client_country_code c;
       { (impl_client_info c.client_client) with
         P.client_network = network.network_num;
         P.client_kind = Known_location (Ip.from_name c.client_hostname,
           c.client_port);
+        P.client_country_code = c.client_country_code;
         P.client_state = client_state (as_client c);
         P.client_type = client_type c;
         P.client_name = (Printf.sprintf "%s:%d"
@@ -123,7 +125,8 @@ let _ =
         P.client_num = (client_num (as_client c));
         P.client_connect_time = BasicSocket.last_time ();
         P.client_software = c.client_software;
-        P.client_downloaded = c.client_downloaded;
+        P.client_total_downloaded = c.client_total_downloaded;
+        P.client_session_downloaded = c.client_session_downloaded;
       }
   );
     client_ops.op_client_bprint <- (fun c buf ->
@@ -141,8 +144,8 @@ let _ =
         client_print cc o;
         Printf.bprintf buf "client: %s downloaded: %s uploaded: %s"
           cinfo.GuiTypes.client_software
-          (Int64.to_string cinfo.GuiTypes.client_downloaded)
-        (Int64.to_string cinfo.GuiTypes.client_uploaded);
+          (Int64.to_string cinfo.GuiTypes.client_total_downloaded)
+        (Int64.to_string cinfo.GuiTypes.client_total_uploaded);
         Printf.bprintf buf "\nfilename: %s\n\n" info.GuiTypes.file_name;
     );
     client_ops.op_client_dprint_html <- (fun c o file str ->
@@ -155,7 +158,9 @@ let _ =
 	    Indirect_location (_, _, ip, port)
 	  | Known_location (ip, port) -> ip, port
 	in
-        let ccode,cname = Geoip.get_country (fst (client_ip,client_port)) in
+        let ccode,cname =
+          Geoip.get_country_code_name cinfo.GuiTypes.client_country_code
+        in
 
         Printf.bprintf buf " \\<tr onMouseOver=\\\"mOvr(this);\\\"
     onMouseOut=\\\"mOut(this);\\\" class=\\\"%s\\\"\\>" str;
@@ -182,9 +187,11 @@ let _ =
           ("", "sr", "D");
           ("", "sr", "N");
           ("", "sr", Printf.sprintf "%s:%d" (Ip.to_string client_ip) client_port);
-          ] @ (if !Geoip.active then [(cname, "sr", ccode)] else []) @ [
-          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_uploaded));
-          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_downloaded));
+          ] @ (if Geoip.active () then [(cname, "sr", CommonPictures.flag_html ccode)] else []) @ [
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_total_uploaded));
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_total_downloaded));
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_session_uploaded));
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_session_downloaded));
           ("", "sr", info.GuiTypes.file_name); ]);
         true
     )
@@ -416,6 +423,17 @@ let _ =
       | _ -> ()
     ) file.file_clients
   );
+  file_ops.op_file_download_order <- (fun file strategy ->
+      match file.file_swarmer with
+      | None -> None
+      | Some s ->
+          (match strategy with
+          (* return current strategy *)
+          | None -> Some (CommonSwarming.get_strategy s)
+          | Some strategy -> CommonSwarming.set_strategy s strategy;
+                      Some (CommonSwarming.get_strategy s))
+  );
+  file_ops.op_file_queue <- file_ops.op_file_pause;
   file_ops.op_file_resume <- (fun file -> ());
   file_ops.op_file_print <- (fun file buf -> ());
   network.op_network_close_search <- (fun s -> ());
